from pathlib import Path
from typing import Union


class Tagger:
    def __init__(self, protocol_table_path: Union[Path, str]):
        if type(protocol_table_path) is str:
            self.protocol_table_path = Path(protocol_table_path)
            self.table_protocols = pd.read_csv(
                protocol_table_path, sep="\t", index_col=False
            )
        else:
            self.protocol_table_path = protocol_table_path
            self.table_protocols = pd.read_csv(
                protocol_table_path, sep="\t", index_col=False
            )

    def classification_by_min_max(self, json: dict) -> list:
        pass



import logging
from logging import Logger
from pathlib import Path
from typing import Optional, Union


class ColorFormatter(logging.Formatter):
    def __init__(self, format: str):
        blue = "\x1b[94;20m"
        yellow = "\x1b[33;20m"
        red = "\x1b[31;20m"
        reset = "\x1b[0m"

        self.formats = {
            logging.DEBUG: blue + format + reset,
            logging.INFO: format,
            logging.WARNING: yellow + format + reset,
            logging.ERROR: red + format + reset,
        }

    def format(self, record):
        log_fmt = self.formats.get(record.levelno)
        formatter = logging.Formatter(log_fmt, datefmt="%H:%M:%S")
        return formatter.format(record)


def set_logger(
    level: int = logging.INFO, outpath: Optional[Union[Path, str]] = None
) -> Logger:
    logger = logging.getLogger("dcm2mids")
    logger.setLevel(logging.DEBUG)

    # Create console handler and set level desired level
    ch = logging.StreamHandler()
    ch.setLevel(level)

    format_string = "[%(asctime)s] %(name)s - %(levelname)s: %(message)s"

    # Create colored formatter
    color_formatter = ColorFormatter(format_string)
    ch.setFormatter(color_formatter)
    logger.addHandler(ch)

    if outpath:
        fh = logging.FileHandler(outpath, mode="w")
        fh.setLevel(logging.DEBUG)
        formatter = logging.Formatter(format_string)
        fh.setFormatter(formatter)
        logger.addHandler(fh)

    return logger



from pydicom.dataset import Dataset
from pydicom.datadict import get_entry


def convert_string(input_string: str) -> str:
    """Split the string into words, capitalize each word, and then join them without spaces"""
    cleaned_string = input_string.replace('(', ' ').replace(')', ' ')
    return ''.join(word.capitalize() for word in cleaned_string.split())

def dictify(ds: Dataset, stop_before_pixels: bool = True) -> dict:
    """Turn a pydicom Dataset into a dict with keys derived from the Element names."""
    output = dict()
    for elem in ds:
        if elem.name == "Pixel Data" and stop_before_pixels:
            continue
        if elem.VR != "SQ":
            if not elem.tag.is_private:
                output[get_entry(elem.tag)[-1]] = str(elem.value)
            else:
                output[convert_string(elem.name)] = str(elem.value)
        else:
            if not elem.tag.is_private:
                output[get_entry(elem.tag)[-1]] = [dictify(item) for item in elem]
            else:
                output[convert_string(elem.name)] = [dictify(item) for item in elem]  
    return output



import logging
from pathlib import Path
from typing import List, Tuple

from pydicom import Dataset
from pydicom.fileset import FileInstance

from ..procedures import Procedures

logger = logging.getLogger("dcm2mids").getChild("tomography_procedure")


class TomographyProcedures(Procedures):
    """Conversion logic for General Radiologic Imaging procedures."""

    def __init__(
        self,
        mids_path: Path,
        bodypart: str,
        use_bodypart: bool,
        use_viewposition: bool,
    ):
        super().__init__(mids_path, bodypart, use_bodypart, use_viewposition)
        self.scans_header = [
                'Filename',
                'BodyPart',
                'ViewPosition',
                'SeriesNumber',
                'AccessionNumber',
                'Manufacturer',
                'ManufacturerModelName',
                'Modality',
                'Columns',
                'Rows',
                'PhotometricInterpretation',
                'Laterality',
                'KVP',
                'Exposure',
                'ExposureTime',
                'XRayTubeCurrent'
            ]

    def classify_image_type(
        self, instance: FileInstance
    ) -> Tuple[str, Tuple[str, ...]]:
        """
        Classifies an image based on its modality.

        :param instance: The instance to be classified.
        :type instance: pydicom.fileset.FileInstance
        :returns: A tuple containing the image type and a tuple of labels for that type.
        :rtype: tuple[str, tuple[str, ...]]
        """
        if instance.Modality in ["CR", "DX"]:
            self.scans_header = [
                "Filename",
                "BodyPart",
                "ViewPosition",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "Laterality",
                "KVP",
                "Exposure",
                "ExposureTime",
                "XRayTubeCurrent",
            ]
            return (
                instance.Modality.lower(),
                (
                    (instance.Modality.lower(),)
                    if self.bodypart in ["head", "brain", "skull"]
                    else (
                        "mim-rx",
                        instance.Modality.lower(),
                    )
                ),
                ".png",
            )

        if instance.Modality in ["CT"]:
            self.scans_header = [
                "ScanFile",
                "BodyPart",
                "ViewPosition",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "Laterality",
                "KVP",
                "Exposure" "ExposureTime",
                "XRayTubeCurrent",
                "DataCollectionDiameter",
                "ReconstructionDiameter",
                "SliceThickness",
                "ConvolutionKernel",
                "ReconstructionAlgorithm",
                "DistanceSourceToDetector" "Image Orientation",
                "SmallestImagePixelValue",
                "LargestImagePixelValue",
                "WindowCenter",
                "WindowWidth",
            ]
            return (
                instance.Modality.lower(),
                (
                    (instance.Modality.lower(),)
                    if self.bodypart in ["head", "brain", "skull"]
                    else (
                        "mim-rx",
                        instance.Modality.lower(),
                    )
                ),
                ".nii.gz",
            )
        return ("", tuple(), "")

    def get_name(
        self, dataset: Dataset, modality: str, mim: Tuple[str, ...]
    ) -> Tuple[Path, Path]:
        """
        Generates a name for the image based on its metadata.

        :param dataset: The dataset containing the image metadata.
        :type dataset: pydicom.Dataset
        :param modality: The modality of the image.
        :type modality: str
        :param mim: A tuple of labels to be included in the filepath.
        :type mim: tuple[str, ...]
        :returns: A Path object representing the generated name.
        :rtype: pathlib.Path
        """

        sub = f"sub-{dataset.PatientID}"
        ses = f"ses-{dataset.StudyID}"
        run = (
            f"run-{dataset.SeriesNumber}"
            if dataset.data_element("SeriesNumber")
            else ""
        )
        if self.use_bodypart:
            bp = (
                f"bp-{dataset.BodyPartExamined}"
                if "BodyPartExamined" in dataset
                else f"bp-{self.bodypart}"
            )
        else:
            bp = ""
        lat = f"lat-{dataset.Laterality}" if dataset.data_element("Laterality") else ""
        vp = (
            f"vp-{convert_orientation(dataset.data_element('ImageOrientationPatient'))[0]}"
            if dataset.data_element("ImageOrientationPatient")
            else ""
        )
        chunk = (
            f"chunk-{dataset.InstanceNumber}"
            if dataset.data_element("InstanceNumber") and self.use_chunk
            else ""
        )
        mod = modality
        filename = "_".join(
            [part for part in [sub, ses, run, bp, lat, vp, chunk, mod] if part != ""]
        )
        return (
            self.mids_path.joinpath(sub, ses, *mim, filename),
            self.mids_path.joinpath(sub, ses),
        )

    def convert_to_image(self, instance: FileInstance, file_path_mids: Path):
        """
        Converts a DICOM to an image.

        :param instance: The DICOM image instance.
        :type instance: pydicom.fileset.FileInstance
        :param file_path_mids: The path where the converted image will be saved.
        :type file_path_mids: pathlib.Path
        """
        if "".join(file_path_mids.suffixes) == ".nii.gz":
            pass
        else:
            file_path_mids.parent.mkdir(parents=True, exist_ok=True)
            image = sitk.ReadImage(instance.path)
            sitk.WriteImage(image, file_path_mids)


def run(self, instance_list: List[Tuple[int, FileInstance]]):
    """
    Runs the image conversion pipeline on a list of instances.

    :param instance_list: A list of tuples containing the instance number and the DICOM instance.
    :type instance_list: list[uuple[int, pydicom.fileset.FileInstance]]
    """

    self.use_chunk = len(instance_list) > 1
    list_scan_metadata = []
    for _, instance in sorted(instance_list, key=lambda x: x[0]):
        print(instance)
        dataset = instance.load()
        modality, mim, ext = self.classify_image_type(instance)
        file_path_mids, session_absolute_path_mids = self.get_name(
            dataset, modality, mim
        )

        self.convert_to_image(instance, file_path_mids.with_suffix(ext))
        self.convert_to_jsonfile(dataset, file_path_mids.with_suffix(".json"))
        file_path_relative_mids = file_path_mids.relative_to(
            session_absolute_path_mids
        ).with_suffix(".png")
        list_scan_metadata.append(
            self.get_scan_metadata(dataset, file_path_relative_mids)
        )
    return list_scan_metadata


def convert_orientation(orientation):
    mapping = {
        "1\\0\\0\\0\\1\\0": ("Sag", "LAS"),  # Left-Anterior-Superior
        "0\\1\\0\\0\\0\\-1": ("Cor", "RAS"),  # Right-Anterior-Superior
        "1\\0\\0\\0\\0\\-1": ("Ax", "PAS"),  # Posterior-Anterior-Superior
        # Add more mappings as needed
    }
    return mapping.get(orientation, ("Unknown", "Unknown"))



import logging
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Union

import pandas as pd
from pydicom.fileset import FileSet

logger = logging.getLogger("dcm2mids").getChild("generate_tsvs")

participants_header = [
    "participant_id",  # This is the participant_id from the BIDS/MIDS standard (sub-<participant_id>)
    "participant_pseudo_id",  # This is the participant_pseudo_id from the PatientID (participant_id)
    "sex",  # This is the gender of the participant
    "participant_birthday",  # This is the patient's birthday
    "ages",  # This is the list of ages of the participant in the sessions
    "modalities",  # This is the list of modalities used in the sessions
    "body_parts_examined",  # This is the list of body parts examined in the sessions
]

session_header = [
    "session_id",  # This is the session_id from the BIDS/MIDS standard (ses-<session_id>)
    "session_pseudo_id",  # This is the session_pseudo_id from the PatientID (session_id)
    "acq_time",  # This is the acquisition date time of the session
    "age",  # this is the age of the participant in the session
]


def get_session_row(fileset: FileSet, subject: str, session: str) -> Dict[str, str]:
    """
    Generate a row for the Sessions TSV file.

    :param fileset: The FileSet to search for the subject and session.
    :type fileset: pydicom.fileset.FileSet
    :param subject: The ID of the subject this session belongs to.
    :type subject: str
    :param session: The ID of the session.
    :type session: str
    :return: A dictionary containing the session row data.
    :rtype: dict
    """

    session_row = fileset.find_values(
        ["StudyDate", "StudyTime", "AcquisitionDateTime", "AcquisitionDate", "AcquisitionTime", "PatientBirthDate"],
        fileset.find(PatientID=subject, StudyID=session),
        load=True,
    )
    session_row["session_id"] = f"sub-{session}"
    session_row["session_pseudo_id"] = session
    logger.debug("AcquisitionDateTime: %s", session_row.get("AcquisitionDateTime"))
    logger.debug("AcquisitionDate: %s", session_row.get("AcquisitionDate"))
    logger.debug("AcquisitionTime: %s", session_row.get("AcquisitionTime"))
    logger.debug("StudyDate: %s", session_row.get("StudyDate"))
    logger.debug("StudyTime: %s", session_row.get("StudyTime"))

    date_time = session_row["StudyDate"][0] + session_row["StudyTime"][0]
    if "." not in date_time:
        date_time += ".000000"
    session_row["session_date_time"] = [date_time]
    session_row.pop("StudyDate")
    session_row.pop("StudyTime")
    session_row["session_date_time"] = [
        datetime.strptime(s, "%Y%m%d%H%M%S.%f")
        for s in session_row["session_date_time"]
    ][0]
    if session_row.get("PatientBirthDate")[0]:

        session_row["PatientBirthDate"] = datetime.strptime(
            session_row["PatientBirthDate"][0], "%Y%m%d"
        )
        patient_age = (
            session_row["session_date_time"] - session_row["PatientBirthDate"]
        ).days // 365.25

        session_row["PatientBirthDate"] = session_row["PatientBirthDate"].strftime(
            "%Y-%m-%d"
        )
    else:
        session_row["PatientBirthDate"] = "n/a"
        patient_age = "n/a"
    
    session_row["acq_time"] = session_row["session_date_time"].isoformat()
    session_row["age"] = patient_age

    return session_row


def get_participant_row(
    participant: Dict[str, Union[str, list]],
    fileset: FileSet,
    subject: str,
    bodypart: str,
    participant_birthday: str,
) -> Dict[str, Union[str, list]]:
    """
    Generate a row for the Participants TSV file.

    :param participant: A dictionary with participant data.
    :type participant: dict
    :param fileset: The FileSet to search for the subject and session.
    :type fileset: pydicom.fileset.FileSet
    :param subject: The ID of the subject this session belongs to.
    :type subject: str
    :param bodypart: The bodypart(s) contained in the dataset for this participant.
    :type bodypart: str
    :param participant_birthday: The participant's birthday.
    :type participant_birthday: str
    :return: A dictionary containing the participant row data.
    :rtype: dict
    """

    participant["participant_id"] = f"sub-{subject}"
    participant["participant_pseudo_id"] = subject
    participant["ages"] = list(set(participant["ages"]))
    participant.update(
        fileset.find_values(
            ["Modality", "BodyPartExamined", "PatientSex"],
            fileset.find(PatientID=subject),
            load=True,
        )
    )
    participant["sex"] = list(set(participant.pop("PatientSex")))[0]
    if len(participant["BodyPartExamined"]) == 0:
        participant["BodyPartExamined"] = [bodypart]
    participant["participant_birthday"] = participant_birthday
    participant["modalities"] = participant.pop("Modality")
    participant["body_parts_examined"] = participant.pop("BodyPartExamined")
    return participant


def save_session_tsv(sessions: List[Dict[str, str]], mids_path: Path, subject: str):
    """
    Save the sessions for a given subject to a TSV file in the MIDS directory.

    :param sessions: The list of sessions to save.
    :type sessions: list[dict]
    :param mids_path: The path to the MIDS directory.
    :type mids_path: pathlib.Path
    :param subject: The ID of the subject for which to save the sessions.
    :type subject: str
    """

    session_tsv = mids_path.joinpath(f"sub-{subject}", f"sub-{subject}_sessions.tsv")
    df = pd.DataFrame(sessions)[session_header]
    df = df.sort_values("acq_time", ascending=False)  # type: ignore
    session_tsv.parent.mkdir(parents=True, exist_ok=True)
    df.to_csv(session_tsv, sep="\t", index=False)


def save_participant_tsv(
    participants: List[Dict[str, Union[str, list]]], mids_path: Path
):
    """
    Save the participants to a TSV file in the MIDS directory.

    :param participants: The list of participants to save.
    :type participants: List[Dict]
    :param mids_path: The path to the MIDS directory.
    :type mids_path: Path
    """

    participant_tsv = mids_path.joinpath("participants.tsv")
    df = pd.DataFrame(participants)[participants_header]
    df = df.sort_values("participant_id", ascending=False)  # type: ignore
    participant_tsv.parent.mkdir(parents=True, exist_ok=True)
    df.to_csv(participant_tsv, sep="\t", index=False)


def save_scans_tsv(
    scans: List[Dict[str, str]], mids_path: Path, subject: str, session: str
):
    """
    Save the scans for a given subject and session to a TSV file in the MIDS directory.

    :param scans: The list of scans to save.
    :type scans: List[Dict]
    :param mids_path: The path to the MIDS directory.
    :type mids_path: Path
    :param subject: The ID of the subject for which to save the scans.
    :type subject: str
    :param session: The ID of the session for which to save the scans.
    :type session: str
    """

    scan_tsv = mids_path.joinpath(
        f"sub-{subject}",
        f"ses-{session}",
        f"sub-{subject}_ses-{session}_scans.tsv",
    )
    df = pd.DataFrame(scans)
    df = df.sort_values("scan_file", ascending=False)
    scan_tsv.parent.mkdir(parents=True, exist_ok=True)
    df.to_csv(scan_tsv, sep="\t", index=False)



import logging
from pathlib import Path
from typing import Tuple, List
from pydicom import Dataset

from pydicom.fileset import FileInstance

from ..procedures import Procedures

logger = logging.getLogger("dcm2mids").getChild("conventional_radiology_procedure")

bids_bp = ["head", "brain", "skull"]

class ConventionalRadiologyProcedures(Procedures):
    """Conversion logic for General Radiologic Imaging procedures."""

    def __init__(
        self,
        mids_path: Path,
        modality: str,
        bodypart: str,
        use_bodypart: bool,
        use_viewposition: bool,
    ):
        super().__init__(mids_path, modality, bodypart, use_bodypart, use_viewposition)
        self.scans_header = [
                "Filename",
                "BodyPart",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "Laterality",
            ]
        self.modality = modality.lower()
        self.bodypart = bodypart
        self.use_bodypart = use_bodypart
        self.use_viewposition = use_viewposition
        self.mim = "mim-rx" if self.bodypart in bids_bp else ""
        self.bids_folder = modality.lower()
        self.extension = ".png"

    def get_name(self, dataset: Dataset) -> Tuple[Path, Path]:
        """
        Generates a name for the image based on its metadata.

        :param dataset: The dataset containing the image metadata.
        :type dataset: pydicom.Dataset
        :param modality: The modality of the image.
        :type modality: str
        :param mim: A tuple of labels to be included in the filepath.
        :type mim: tuple[str, ...]
        :returns: A Path object representing the generated name.
        :rtype: pathlib.Path
        """

        sub = f"sub-{dataset.PatientID}"
        ses = f"ses-{dataset.StudyID}"
        run = (
            f"run-{dataset.SeriesNumber}"
            if dataset.data_element("SeriesNumber")
            else ""
        )
        if self.use_bodypart:
            bp = (
                f"bp-{dataset.BodyPartExamined}"
                if "BodyPartExamined" in dataset
                else f"bp-{self.bodypart}"
            )
        else:
            bp = ""
        lat = f"lat-{dataset.Laterality}" if dataset.data_element("Laterality") else ""
        if self.use_viewposition:
            
            vp = (
                f"vp-{convert_orientation_2D(dataset.data_element('ImageOrientationPatient'))}"
                if dataset.data_element("ImageOrientationPatient")
                else ""
            )
        else:
            vp = ""
        
        chunk = (
            f"chunk-{dataset.InstanceNumber}"
            if dataset.data_element("InstanceNumber") and self.use_chunk
            else ""
        )
        mod = self.modality
        filename = "_".join(
            [part for part in [sub, ses, run, bp, lat, vp, chunk, mod] if part != ""]
        )
        return (
            self.mids_path.joinpath(sub, ses, *mim, filename),
            self.mids_path.joinpath(sub, ses),
        )

    def convert_to_image(self, instance: FileInstance, file_path_mids: Path):
        """
        Converts a DICOM to an image.

        :param instance: The DICOM image instance.
        :type instance: pydicom.fileset.FileInstance
        :param file_path_mids: The path where the converted image will be saved.
        :type file_path_mids: pathlib.Path
        """
        file_path_mids.parent.mkdir(parents=True, exist_ok=True)
        image = sitk.ReadImage(instance.path)
        sitk.WriteImage(image, file_path_mids)

    
    def get_scan_metadata(self, dataset, file_path_mids):
        subs = lambda s: re.sub(r"(?<!^)(?=[A-Z])", "_", s).lower()
        return {
            subs(key): value
            for key, value in zip(
                self.scans_header,
                [
                    str(file_path_mids.with_suffix(".png")),
                    (
                        dataset.BodyPartExamined
                        if "BodyPartExamined" in dataset
                        else self.bodypart
                    ),
                    *[
                        (dataset[i].value if i in dataset else "n/a")
                        for i in self.scans_header[2:]
                    ],
                ],
            )
        }
    

def run(self, instance_list: List[FileInstance]):
        """
        Runs the image conversion pipeline on a list of instances.

        :param instance_list: A list of tuples containing the instance number and the DICOM instance.
        :type instance_list: list[uuple[int, pydicom.fileset.FileInstance]]
        """

        self.use_chunk = len(instance_list) > 1
        list_scan_metadata = []
        for instance in instance_list:
            dataset = instance.load()
            modality, mim, ext = self.classify_image_type(instance)
            file_path_mids, session_absolute_path_mids = self.get_name(
                dataset, modality, mim
            )

            self.convert_to_image(instance, file_path_mids.with_suffix(ext))
            self.convert_to_jsonfile(dataset, file_path_mids.with_suffix(".json"))
            file_path_relative_mids = file_path_mids.relative_to(
                session_absolute_path_mids
            ).with_suffix(".png")
            list_scan_metadata.append(
                self.get_scan_metadata(dataset, file_path_relative_mids)
            )
            logger.info(
                "Successfully processed instance %s",
                instance.path,
            )
            logger.info(
                "Saved to %s",
                file_path_relative_mids.stem,
            )
        return list_scan_metadata

def convert_orientation_2D(orientation):
    mapping = {
        "1\\0\\0\\0\\1\\0": "AP",  # Anterior-Posterior
        "0\\1\\0\\0\\0\\-1": "PA",  # Posterior-Anterior
        "1\\0\\0\\0\\0\\-1": "LAT",  # Lateral
        "1\\0\\0\\0\\1\\1": "OB",  # Oblique
        # Add more mappings as needed
    }
    return mapping.get(orientation, "Unknown")

def convert_orientation(orientation):
    mapping = {
        "1\\0\\0\\0\\1\\0": ("Sag", "LAS"),  # Left-Anterior-Superior
        "0\\1\\0\\0\\0\\-1": ("Cor", "RAS"),  # Right-Anterior-Superior
        "1\\0\\0\\0\\0\\-1": ("Ax", "PAS"),  # Posterior-Anterior-Superior
        # Add more mappings as needed
    }
    return mapping.get(orientation, ("Unknown", "Unknown"))
def convert_orientation_full(orientation):
    mapping = {
        "1\\0\\0\\0\\1\\0": ("Sag", "LAS"),  # Left-Anterior-Superior
        "0\\1\\0\\0\\0\\-1": ("Cor", "RAS"),  # Right-Anterior-Superior
        "1\\0\\0\\0\\0\\-1": ("Ax", "PAS"),  # Posterior-Anterior-Superior
        "0\\0\\-1\\1\\0\\0": ("Ax", "RSP"),  # Right-Superior-Posterior
        "0\\0\\1\\1\\0\\0": ("Ax", "RIP"),  # Right-Inferior-Posterior
        "0\\0\\1\\0\\1\\0": ("Cor", "ASP"),  # Anterior-Superior-Posterior
        "0\\0\\-1\\0\\1\\0": ("Cor", "AIP"),  # Anterior-Inferior-Posterior
        "1\\0\\0\\0\\0\\1": ("Sag", "LAP"),  # Left-Anterior-Posterior
        "1\\0\\0\\0\\0\\-1": ("Sag", "LIP"),  # Left-Inferior-Posterior
        # Add more mappings as needed
    }
    return mapping.get(orientation, ("Unknown", "Unknown"))


import logging
from datetime import datetime
from pathlib import Path
from typing import Union

from pydicom.fileset import FileSet

from .generate_tsvs import *
from .procedures import *

logger = logging.getLogger(__name__)


def create_mids_directory(
    fileset: FileSet, mids_path: Union[Path, str], bodypart: str
) -> None:
    """
    Create the MIDS directory structure for a given file set and body  part.

    :param fileset: The FileSet object containing the data to be processed.
    :type fileset: pydicom.fileset.FileSet
    :param mids_path: The path to the MIDS directory where the data will be stored.
    :type mids_path: Union[pathlib.Path, str]
    :param bodypart: The body part to be processed (e.g., "head", "neck", etc.).
    :type bodypart: str
    :return: None
    :rtype: None
    """

    use_bodypart = len(fileset.find_values("BodyPartExamined", load=True)) > 1
    logger.debug("`BodyPartExamined` tag: %s", use_bodypart)
    use_viewposition = len(fileset.find_values("ViewPosition", load=True)) > 1
    logger.debug("`ViewPosition` tag: %s", use_bodypart)
    mids_path = Path(mids_path)
    participants = []
    for subject in fileset.find_values("PatientID", load=True):
        logger.debug("Subject: %s", subject)
        participant = {}
        sessions = []
        for session in fileset.find_values("StudyID", fileset.find(PatientID=subject, load=True), load=True):
            logger.debug("Session: %s", session)
            scans = []
            for scan in fileset.find_values(
                "SeriesNumber", fileset.find(PatientID=subject, StudyID=session, load=True), load=True
            ):
                logger.debug("Scan: %s", scan)
                instance_list = fileset.find(
                    PatientID=subject, StudyID=session, SeriesNumber=scan,
                    load=True
                )
                instance_list = sorted(
                    instance_list, key=lambda x: int(x.InstanceNumber)
                )
                logger.debug("Number of instances: %d", len(instance_list))
                modality = instance_list[0].Modality
                if modality == "MR":
                    scans_row = MagneticResonanceProcedures(
                        mids_path, bodypart, use_bodypart, use_viewposition
                    ).run(instance_list)
                if modality in ["CR", "DX"]:
                    scans_row = ConventionalRadiologyProcedures(
                        mids_path, bodypart, use_bodypart, use_viewposition
                    ).run(instance_list)
                if modality in ["CT", "PT"]:
                    scans_row = TomographyProcedures(
                        mids_path,  bodypart, use_bodypart, use_viewposition
                    ).run(instance_list)
                if modality in ["OP", "SC", "XC", "OT"]:
                    scans_row = OphthalmographyProcedures(
                        mids_path, bodypart, use_bodypart, use_viewposition
                    ).run(instance_list)
                if modality in ["SM", "BF"]:
                    scans_row = MicroscopyProcedures(
                        mids_path, bodypart, use_bodypart, use_viewposition
                    ).run(instance_list)
                scans.extend(scans_row)
            save_scans_tsv(scans, mids_path, subject, session)  # type: ignore
            logger.debug(
                "%d scans created from session %s in subject %s.",
                len(scans),
                session,
                subject,
            )
            session_row = get_session_row(fileset, subject, session)  # type: ignore
            patient_age = session_row["age"]
            if "age" not in participant:
                participant["ages"] = []
            participant["ages"].append(patient_age)
            participant_birthday = session_row.pop("PatientBirthDate")
            sessions.append(session_row)
        save_session_tsv(sessions, mids_path, subject)  # type: ignore
        logger.debug(
            "%d sessions created from subject %s.",
            len(sessions),
            subject,
        )
        participant = get_participant_row(
            participant, fileset, subject, bodypart, participant_birthday  # type: ignore
        )
        participants.append(participant)
    save_participant_tsv(participants, mids_path)
    logger.debug("%d participants processed.", len(participants))



from datetime import datetime
import logging
import re
from pathlib import Path
from shutil import copyfile
from typing import List, Tuple

import SimpleITK as sitk
from pydicom import Dataset
from pydicom.fileset import FileInstance

from ..procedures import Procedures

logger = logging.getLogger("dcm2mids").getChild("microscopy_procedure")


class MicroscopyProcedures(Procedures):
    """Conversion logic for Visible Light Imaging procedures."""

    def __init__(
        self,
        mids_path: Path,
        bodypart: str,
        use_bodypart: bool,
        use_viewposition: bool,
    ):
        super().__init__(mids_path, bodypart, use_bodypart, use_viewposition)

    def classify_image_type(
        self, instance: FileInstance
    ) -> Tuple[str, Tuple[str, ...], str]:
        """
        Classifies an image based on its modality.

        :param instance: The instance to be classified.
        :type instance: pydicom.fileset.FileInstance
        :returns: A tuple containing the image type and a tuple of labels for that type.
        :rtype: tuple[str, tuple[str, ...]]
        """

        logger.debug("Processing instance %s", instance.path)
        logger.debug("Instance modality: %s", instance.Modality)
        if instance.Modality in ["OP", "SC", "XC", "OT"]:
            self.scans_header = [
                "Filename",
                "BodyPart",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "Laterality",
            ]
            return ("op", ("mim-light", "op"), ".png")
        if instance.Modality in ["BF", "SM"]:
            self.scans_header = [
                "ScanFile",
                "BodyPart",
                "AcquisitionDateTime",
                "SeriesNumber",
                "InstanceNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "ImagedVolumeHeight",
                "ImagedVolumeWidth",
                "NumberOfFrames",
                "note",
            ]
            return ("BF", ("micr",), ".dcm")
        return ("", tuple(), "")

    def get_name(
        self, dataset: Dataset, modality: str, mim: Tuple[str, ...]
    ) -> Tuple[Path, Path]:
        """
        Generates a name for the image based on its metadata.

        :param dataset: The dataset containing the image metadata.
        :type dataset: pydicom.Dataset
        :param modality: The modality of the image.
        :type modality: str
        :param mim: A tuple of labels to be included in the filepath.
        :type mim: tuple[str, ...]
        :returns: A Path object representing the generated name.
        :rtype: pathlib.Path
        """

        sub = f"sub-{dataset.PatientID}"
        ses = f"ses-{dataset.StudyID}"
        run = (
            f"run-{dataset.SeriesNumber}"
            if dataset.data_element("SeriesNumber")
            else ""
        )
        if self.use_bodypart:
            bp = (
                f"bp-{dataset.BodyPartExamined}"
                if "BodyPartExamined" in dataset
                else f"bp-{self.bodypart}"
            )
        else:
            bp = ""
        lat = f"lat-{dataset.Laterality}" if "Laterality" in dataset else ""
        vp = ""  # f"vp-{dataset.data_element('ViewPosition')}" if dataset.data_element("ViewPosition") else ""
        chunk = (
            f"chunk-{dataset.InstanceNumber}"
            if dataset.data_element("InstanceNumber") and self.use_chunk
            else ""
        )
        mod = modality
        filename = "_".join(
            [part for part in [sub, ses, run, bp, lat, vp, chunk, mod] if part != ""]
        )
        return (
            self.mids_path.joinpath(sub, ses, *mim, filename),
            self.mids_path.joinpath(sub, ses),
        )

    def convert_to_image(self, instance: FileInstance, file_path_mids: Path):
        """
        Converts a DICOM to an image.

        :param instance: The DICOM image instance.
        :type instance: pydicom.fileset.FileInstance
        :param file_path_mids: The path where the converted image will be saved.
        :type file_path_mids: pathlib.Path
        """
        file_path_mids.parent.mkdir(parents=True, exist_ok=True)
        if file_path_mids.suffix == ".dcm":
            print("instance.path", instance.path)
            copyfile(instance.path, file_path_mids)
        else:
            file_path_mids.parent.mkdir(parents=True, exist_ok=True)
            image = sitk.ReadImage(instance.path)
            sitk.WriteImage(image, file_path_mids)

    def get_scan_metadata(self, dataset, file_path_mids):
        subs = lambda s: re.sub(r"(?<!^)(?=[A-Z])", "_", s).lower()
        
        return {
            subs(key): value
            for key, value in zip(
                self.scans_header,
                [
                    str(file_path_mids),
                    (
                        dataset.BodyPartExamined
                        if "BodyPartExamined" in dataset
                        else self.bodypart
                    ),
                    datetime.strptime(
                        dataset.AcquisitionDateTime[:14], "%Y%m%d%H%M%S"
                    ).strftime("%Y-%m-%dT%H:%M:%S"),
                    *[
                        (dataset[i].value if i in dataset else "n/a")
                        for i in self.scans_header[3:-1]
                    ],
                    dataset[(0x000b, 0x1010)].value,
                ],
            )
        }
        

    def run(self, instance_list: List[FileInstance]):
        """
        Runs the image conversion pipeline on a list of instances.

        :param instance_list: A list of tuples containing the instance number and the DICOM instance.
        :type instance_list: list[uuple[int, pydicom.fileset.FileInstance]]
        """

        self.use_chunk = len(instance_list) > 1
        list_scan_metadata = []
        for instance in instance_list:
            dataset = instance.load()
            modality, mim, ext = self.classify_image_type(instance)
            file_path_mids, session_absolute_path_mids = self.get_name(
                dataset, modality, mim
            )

            self.convert_to_image(instance, file_path_mids.with_suffix(ext))
            self.convert_to_jsonfile(dataset, file_path_mids.with_suffix(".json"))
            file_path_relative_mids = file_path_mids.relative_to(
                session_absolute_path_mids
            ).with_suffix(ext)
            list_scan_metadata.append(
                self.get_scan_metadata(dataset, file_path_relative_mids)
            )
            logger.info(
                "Successfully processed instance %s",
                instance.path,
            )
            logger.info(
                "Saved to %s",
                file_path_relative_mids.stem,
            )
        return list_scan_metadata



import logging
import re
from pathlib import Path
from typing import List, Tuple

import SimpleITK as sitk
from pydicom import Dataset
from pydicom.fileset import FileInstance

from ..procedures import Procedures

logger = logging.getLogger("dcm2mids").getChild("ophthalmography_procedure")


class OphthalmographyProcedures(Procedures):
    """Conversion logic for Visible Light Imaging procedures."""

    def __init__(
        self,
        mids_path: Path,
        bodypart: str,
        use_bodypart: bool,
        use_viewposition: bool,
    ):
        super().__init__(mids_path, bodypart, use_bodypart, use_viewposition)

    def classify_image_type(
        self, instance: FileInstance
    ) -> Tuple[str, Tuple[str, ...]]:
        """
        Classifies an image based on its modality.

        :param instance: The instance to be classified.
        :type instance: pydicom.fileset.FileInstance
        :returns: A tuple containing the image type and a tuple of labels for that type.
        :rtype: tuple[str, tuple[str, ...]]
        """

        logger.debug("Processing instance %s", instance.path)
        logger.debug("Instance modality: %s", instance.Modality)
        if instance.Modality in ["OP", "SC", "XC", "OT"]:
            self.scans_header = [
                "ScanFile",
                "BodyPart",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "Laterality",
            ]
            return ("op", ("mim-light", "op"))
        if instance.Modality in ["BF", "SM"]:
            self.scans_header = [
                "ScanFile",
                "BodyPart",
                "SeriesNumber",
                "AccessionNumber",
                "Manufacturer",
                "ManufacturerModelName",
                "Modality",
                "Columns",
                "Rows",
                "PhotometricInterpretation",
                "ImagedVolumeHeight",
                "ImagedVolumeWeight",
                "NumberOfFrames"
            ]
            return ("BF", ("micr",))
        return ("", tuple())

    def get_name(
        self, dataset: Dataset, modality: str, mim: Tuple[str, ...]
    ) -> Tuple[Path, Path]:
        """
        Generates a name for the image based on its metadata.

        :param dataset: The dataset containing the image metadata.
        :type dataset: pydicom.Dataset
        :param modality: The modality of the image.
        :type modality: str
        :param mim: A tuple of labels to be included in the filepath.
        :type mim: tuple[str, ...]
        :returns: A Path object representing the generated name.
        :rtype: pathlib.Path
        """

        sub = f"sub-{dataset.PatientID}"
        ses = f"ses-{dataset.StudyID}"
        run = (
            f"run-{dataset.SeriesNumber}"
            if dataset.data_element("SeriesNumber")
            else ""
        )
        if self.use_bodypart:
            bp = (
                f"bp-{dataset.BodyPartExamined}"
                if "BodyPartExamined" in dataset
                else f"bp-{self.bodypart}"
            )
        else:
            bp = ""
        lat = f"lat-{dataset.Laterality}" if dataset.data_element("Laterality") else ""
        vp = ""  # f"vp-{dataset.data_element('ViewPosition')}" if dataset.data_element("ViewPosition") else ""
        chunk = (
            f"chunk-{dataset.InstanceNumber}"
            if dataset.data_element("InstanceNumber") and self.use_chunk
            else ""
        )
        mod = modality
        filename = "_".join(
            [part for part in [sub, ses, run, bp, lat, vp, chunk, mod] if part != ""]
        )
        return (
            self.mids_path.joinpath(sub, ses, *mim, filename),
            self.mids_path.joinpath(sub, ses),
        )

    def convert_to_image(self, instance: FileInstance, file_path_mids: Path):
        """
        Converts a DICOM to an image.

        :param instance: The DICOM image instance.
        :type instance: pydicom.fileset.FileInstance
        :param file_path_mids: The path where the converted image will be saved.
        :type file_path_mids: pathlib.Path
        """

        file_path_mids.parent.mkdir(parents=True, exist_ok=True)
        image = sitk.ReadImage(instance.path)
        sitk.WriteImage(image, file_path_mids)

    def get_scan_metadata(self, dataset, file_path_mids):
        subs = lambda s: re.sub(r"(?<!^)(?=[A-Z])", "_", s).lower()
        return {
            subs(key): value
            for key, value in zip(
                self.scans_header,
                [
                    str(file_path_mids.with_suffix(".png")),
                    (
                        dataset.BodyPartExamined
                        if "BodyPartExamined" in dataset
                        else self.bodypart
                    ),
                    *[
                        (dataset[i].value if i in dataset else "n/a")
                        for i in self.scans_header[2:]
                    ],
                ],
            )
        }

    def run(self, instance_list: List[FileInstance]):
        """
        Runs the image conversion pipeline on a list of instances.

        :param instance_list: A list of tuples containing the instance number and the DICOM instance.
        :type instance_list: list[uuple[int, pydicom.fileset.FileInstance]]
        """

        self.use_chunk = len(instance_list) > 1
        list_scan_metadata = []
        for instance in instance_list:
            dataset = instance.load()
            modality, mim = self.classify_image_type(instance)
            file_path_mids, session_absolute_path_mids = self.get_name(
                dataset, modality, mim
            )
            self.convert_to_image(instance, file_path_mids.with_suffix(".png"))
            self.convert_to_jsonfile(dataset, file_path_mids.with_suffix(".json"))
            file_path_relative_mids = file_path_mids.relative_to(
                session_absolute_path_mids
            ).with_suffix(".png")
            list_scan_metadata.append(
                self.get_scan_metadata(dataset, file_path_relative_mids)
            )
            logger.info(
                "Successfully processed instance %s",
                instance.path,
            )
            logger.info(
                "Saved to %s",
                file_path_relative_mids.stem,
            )
        return list_scan_metadata



import json
import logging
from abc import ABC, abstractmethod
from pathlib import Path

from pydicom import Dataset

from .dictify import dictify

logger = logging.getLogger("dcm2mids").getChild("procedures")
class Procedures(ABC):
    def __init__(
        self,
        mids_path: Path,
        bodypart: str,
        use_bodypart: bool,
        use_viewposition: bool,
    ):
        self.mids_path = mids_path
        self.bodypart = bodypart
        self.use_bodypart = use_bodypart
        self.use_viewposition = use_viewposition
        self.use_chunk: bool

    # @abstractmethod
    # def control_session_image(self):
    #     pass

    @abstractmethod
    def get_name(self):
        pass

    @abstractmethod
    def run(self):
        pass
    
    @staticmethod
    def convert_to_jsonfile(dataset: Dataset, file_path_mids: Path):
        """
        Convert a dataset to a JSON file.

        :param dataset: The dataset to be converted.
        :type dataset: pydicom.Dataset
        :param file_path_mids: The path to the JSON file where the data will be saved.
        :type file_path_mids: pathlib.Path
        :return: None
        :rtype: None
        """
        json_dict = dictify(dataset)
        file_path_mids.parent.mkdir(parents=True, exist_ok=True)
        with open(file_path_mids, "w") as f:
            json.dump(json_dict, f, indent=4)

    



from pathlib import Path
from typing import Union


def dicom2nifti(
    json_files: Union[str, Path, list], path_to_save: Union[str, Path]
) -> Path:
    folder_nifti = folder_json.parent.parent.joinpath("LOCAL_NIFTI", "files")
    folder_nifti.mkdir(parents=True, exist_ok=True)
    d2n.convert_directory(str(folder_json), str(folder_nifti))
    shutil.copy2(str(folder_json.joinpath("bids.json")), str(folder_nifti.parent))
    if folder_json.joinpath("note.txt").exists():
        shutil.copy2(str(folder_json.joinpath("note.txt")), str(folder_nifti.parent))
    return folder_nifti.parent



import logging

from dcm2mids.procedures.dictify import dictify

from ..procedures import Procedures

logger = logging.getLogger("dcm2mids").getChild("magnetic_resonance_procedure")


class MagneticResonanceProcedures(Procedures):
    def __init__(self, mids_path, bodypart):
        super().__init__(mids_path, bodypart)
        self.reset()

    def reset(self):
        self.dicom_dict = {}

    def generate_metadata(self, instance_list: list):
        """Generate metadata from dicom files."""
        dicom_jsons = []
        ordered_instance_list = [
            instance[1] for instance in sorted(instance_list, key=lambda x: x[0])
        ]
        for instance in ordered_instance_list:
            dicom_jsons.append(dictify(instance.load()))
        for item in zip(*[items for items in [json.items() for json in dicom_jsons]]):
            values = [i[1] for i in item]
            print(values)
            if len(set(values)) == 1:
                dicom_dict[item[0][0]] = item[0][1]
            else:
                dicom_dict[item[0][0]] = values
            print(dicom_dict[item[0][0]])

    def classify_image_type(self):
        pass

    def control_session_image(self):
        pass

    def get_name(self):
        pass

    def run(self, instance_list: list):
        self.generate_metadata(instance_list)
        self.classify_image_type()
        self.control_session_image()
        self.get_name()



from pathlib import Path
from shutil import copyfile
from tempfile import TemporaryDirectory

import pytest
from pydicom.data import get_testdata_file
from pydicom.fileset import FileSet

from dcm2mids.get_dicomdir import get_dicomdir

TEST_DICOMDIR = Path(get_testdata_file("DICOMDIR")).parent  # type: ignore
TEST_CT_DICOM = Path(get_testdata_file("CT_small.dcm"))  # type: ignore
TEST_MR_DICOM = Path(get_testdata_file("MR_small.dcm"))  # type: ignore


@pytest.fixture
def tmp_flat_directory():
    tmp = TemporaryDirectory()
    tmp_path = Path(tmp.name)
    copyfile(TEST_CT_DICOM, tmp_path / TEST_CT_DICOM.name)
    copyfile(TEST_MR_DICOM, tmp_path / TEST_MR_DICOM.name)
    yield tmp_path
    tmp.cleanup()


@pytest.fixture
def tmp_nested_directory():
    tmp = TemporaryDirectory()
    tmp_path = Path(tmp.name)
    tmp_path.joinpath("CT").mkdir()
    tmp_path.joinpath("MR").mkdir()
    copyfile(TEST_CT_DICOM, tmp_path.joinpath("CT", TEST_CT_DICOM.name))
    copyfile(TEST_MR_DICOM, tmp_path.joinpath("MR", TEST_MR_DICOM.name))
    yield tmp_path
    tmp.cleanup()


def test_get_dicomdir_with_empty_dir():
    # Create a temporary directory
    with TemporaryDirectory() as tmp:
        with pytest.raises(RuntimeError):
            # Call the get_dicomdir function
            result = get_dicomdir(tmp)


def test_get_dicomdir_with_existing_dicomdir():
    # Call the get_dicomdir function
    result = get_dicomdir(TEST_DICOMDIR)

    # Assert that the result is a FileSet object
    assert isinstance(result, FileSet)

    # Assert that the FileSet is not empty
    assert len(result) > 0


def test_get_dicomdir_with_invalid_argument():
    # Call the get_dicomdir function with an invalid argument
    with pytest.raises(TypeError):
        get_dicomdir(5)  # type: ignore


def test_get_dicomdir_with_no_dicomdir_flat(tmp_flat_directory):
    # Call the get_dicomdir function
    result = get_dicomdir(tmp_flat_directory)

    # Assert that the result is a FileSet object
    assert isinstance(result, FileSet)

    # Assert that the FileSet contains the DICOM files
    assert len(result) == 2


def test_get_dicomdir_with_no_dicomdir_nested(tmp_nested_directory):
    # Call the get_dicomdir function
    result = get_dicomdir(tmp_nested_directory)

    # Assert that the result is a FileSet object
    assert isinstance(result, FileSet)

    # Assert that the FileSet contains the DICOM files
    assert len(result) == 2


def test_get_dicomdir_with_nonexistent_folder():
    # Call the get_dicomdir function with a nonexistent folder
    with pytest.raises(FileNotFoundError):
        get_dicomdir("/path/to/nonexistent/folder")



import logging
from pathlib import Path
from typing import List, Union
from datetime import datetime

from pydicom import dcmread
from pydicom.fileset import FileSet

logger = logging.getLogger(__name__)


def get_dicomdir(input_dir: Union[Path, str], exclude_paths: List[Union[Path, str]] = None) -> FileSet:
    """
    Get the DICOM structure from the input directory.

    :param input_dir: The input directory as a Path object or a string.
    :type input_dir: Union[pathlib.Path, str]
    :raises TypeError: If the input_dir is not a Path object or a string.
    :raises FileNotFoundError: If the input_dir does not exist or is not a directory.
    :return: A FileSet object containing the DICOM files from the input directory.
    :rtype: pydicom.fileset.FileSet
    """
    if not isinstance(input_dir, Path):
        input_dir = Path(input_dir)
    if not input_dir.exists():
        logger.error("%s does not exist.", input_dir)
        raise FileNotFoundError(f"{input_dir} does not exist.")
    if exclude_paths is not None:
        exclude_paths = [Path(p) if not isinstance(p, Path) else p for p in exclude_paths]
    
    dicomdir = input_dir / "DICOMDIR"
    if (
        dicomdir.exists()
    ):  # If DICOMDIR exists, we will use it to get the DICOM structure.
        logger.info("DICOMDIR file found")
        ds = dcmread(dicomdir)
        fs = FileSet(ds)
    elif (
        input_dir.is_dir()
    ):  # If DICOMDIR does not exist, we will search for DICOM files in the input dir.
        fs = FileSet()
        logger.info(
            "DICOMDIR file not found. Listing all `.dcm` files on the directory."
        )
        suffix = "*.[dD][cC][mM]"
        for filename in input_dir.rglob(suffix):
            
            
            if exclude_paths and any(filename.is_relative_to(exclude_path) for exclude_path in exclude_paths): continue
            ds = dcmread(filename)
            txt_file = filename.parent / "note.txt"
            txt_content = ""
            if txt_file.exists():
                with txt_file.open('r') as file:
                    txt_content = file.read()
            if txt_content == "": txt_content = "n/a"
            
            # Reserve a private tag block
            private_creator_tag = 0x000b  # (gggg,00XX) for reserving block
            block = ds.private_block(private_creator_tag, 'Note', create=True)

            # Define the private tag (for example, in the reserved block)
            private_tag = 0x10  # (gggg,xxYY) where xxYY is within the reserved block

            # Add the private tag with VR (e.g., LO for Long String) and value
            block.add_new(private_tag, 'LO', txt_content)
            
            if not ds.StudyID:
                logger.warning(
                    "`StudyID` tag not found for file %s. `AccessionNumber` will be used instead.",
                    filename,
                )
                ds.StudyID = ds.AccessionNumber
            if not ds.StudyTime:
                logger.warning(
                    "`StudyTime` tag not found for file %s. Time part of `AdquisitionDateTime` will be used instead.",
                    filename,
                )
                ds.StudyTime = datetime.strptime(
                    ds.AcquisitionDateTime[:14], "%Y%m%d%H%M%S"
                ).strftime("%H%M%S")
            if not ds.SeriesNumber:
                logger.warning(
                    "`SeriesNumber` tag not found for file %s. `InstanceNumber` will be used instead.",
                    filename,
                )
                ds.SeriesNumber = ds.InstanceNumber
            # try:
            fs.add(ds)
            # except ValueError as e:
            #     print(e)
            #     continue

    else:
        logger.error("%s is not a directory.", input_dir)
        raise NotADirectoryError(f"{input_dir} is not a directory.")
    if len(fs) == 0:
        logger.error("No DICOMDIR/DICOM files found in %s.", input_dir)
        raise RuntimeError(f"No DICOM files found in {input_dir}.")
    logger.info("FileSet has %d elements", len(fs))
    return fs



import argparse
import logging
from pathlib import Path

from .create_mids_directory import create_mids_directory
from .get_dicomdir import get_dicomdir
from .logger import set_logger

parser = argparse.ArgumentParser(
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description="Script for reading a folder with images and converting them to a BIDS/MIDS Structure.",
)

parser.add_argument(
    "-i", "--input", type=Path, help="Path to the input folder", required=True
)
parser.add_argument(
    "-o", "--output", type=Path, help="Path to the output folder", required=True
)
parser.add_argument(
    "-bp",
    "--body-part",
    dest="body_part",
    type=str,
    help="Specify which part of the body is in the dataset",
    required=True,
)
parser.add_argument(
    "-b",
    "--bids",
    dest="bids",
    action="store_true",
    help="Use BIDS standard. Only applicable for protocols/body parts considered in BIDS.",
)
parser.add_argument(
    "-v",
    "--verbose",
    dest="verbose",
    choices=["DEBUG", "INFO", "WARNING", "ERROR"],
    default="INFO",
    help="Verbose level. One of DEBUG, INFO, WARNING, ERROR",
)
parser.add_argument(
    "-log", "--logfile", type=Path, help="Path to the file to store logs"
)
args = parser.parse_args()

log_level = getattr(logging, args.verbose)
root_logger = set_logger(level=log_level, outpath=args.logfile)

fileset = get_dicomdir(args.input)

create_mids_directory(fileset, args.output, args.body_part)

# generate_tsvs(args.output)
